import { randomIntInRange } from '../utils/Random';

interface HarvesterMemory extends CreepMemory {
  sourceNum: number | undefined;
}

export function runAsHarvester(creep: Creep) {
  const memory = creep.memory as HarvesterMemory;

  if (memory.sourceNum === undefined) {
    const sources = creep.room.find(FIND_SOURCES);
    memory.sourceNum = randomIntInRange(0, sources.length);
  }

  if (creep.store.getFreeCapacity() > 0) {
    let sources = creep.room.find(FIND_SOURCES);
    const sourceNum = memory.sourceNum;
    if (creep.harvest(sources[sourceNum]) === ERR_NOT_IN_RANGE) {
      creep.say('To source ' + sourceNum);
      creep.moveTo(sources[sourceNum], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  } else {
    let targets = creep.room.find(FIND_STRUCTURES, {
      filter: (structure) => {
        return (
          (structure.structureType === STRUCTURE_EXTENSION ||
            structure.structureType === STRUCTURE_SPAWN ||
            structure.structureType === STRUCTURE_TOWER) &&
          structure.store.getFreeCapacity(RESOURCE_ENERGY) > 0
        );
      }
    });

    if (targets.length > 0) {
      if (creep.transfer(targets[0], RESOURCE_ENERGY) === ERR_NOT_IN_RANGE) {
        creep.say('to target');
        creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
      }
    } else {
      const spawns = creep.room.find(FIND_MY_SPAWNS);
      if (spawns.length > 0) {
        creep.say('to spawn');
        creep.moveTo(spawns[0]);
      }
    }
  }
}
