import { randomIntInRange } from '../utils/Random';

interface UpgraderMemory extends CreepMemory {
  upgrading: boolean | undefined;
  sourceNum: number | undefined;
}

export function runAsUpgrader(creep: Creep) {
  const memory = creep.memory as UpgraderMemory;

  if (memory.upgrading && creep.store[RESOURCE_ENERGY] === 0) {
    memory.upgrading = false;
  }

  if (!memory.upgrading && creep.store.getFreeCapacity() === 0) {
    memory.upgrading = true;
  }

  if (memory.sourceNum === undefined) {
    const sources = creep.room.find(FIND_SOURCES);
    memory.sourceNum = randomIntInRange(0, sources.length);
  }

  if (memory.upgrading) {
    if (creep.room.controller && creep.upgradeController(creep.room.controller) == ERR_NOT_IN_RANGE) {
      creep.say('⚡ upgrade');
      creep.moveTo(creep.room.controller, { visualizePathStyle: { stroke: '#ffffff' } });
    }
  } else {
    const sources = creep.room.find(FIND_SOURCES);
    const sourceNum = memory.sourceNum;
    if (creep.harvest(sources[sourceNum]) == ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest ' + sourceNum);
      creep.moveTo(sources[sourceNum], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}
