import { ErrorMapper } from 'utils/ErrorMapper';
import { runAsHarvester } from './creep/harvester';
import { runAsBuilder } from './creep/builder';
import { runAsUpgrader } from './creep/upgrader';
import { uniqId } from './utils/Identity';

function makeName(role: string): string {
  return uniqId(role + '_');
}

enum CreepRole {
  HARVESTER = 'harvester',
  UPGRADER = 'upgrader',
  BUILDER = 'builder'
}

function makeCreep(role: CreepRole, count: number) {
  const creeps = Object.values(Game.creeps).filter((c) => c.memory.role === role);
  console.log('Make creep ', role, 'need', count, 'has', creeps.length);
  if (creeps.length < count) {
    const firstSpawn = _.first(Object.values(Game.spawns));
    const name = makeName(role);
    const memory = { role: role } as CreepMemory;
    const err = firstSpawn.spawnCreep([WORK, CARRY, MOVE], name, { memory });
    console.log(`Make creep "${role}" err`, err);
  }
}

// When compiling TS to JS and bundling with rollup, the line numbers and file names in error messages change
// This utility uses source maps to get the line numbers and file names of the original, TS source code
export const loop = ErrorMapper.wrapLoop(() => {
  console.log(`Current game tick is ${Game.time}`);

  // Automatically delete memory of missing creeps
  for (const name in Memory.creeps) {
    if (!(name in Game.creeps)) {
      delete Memory.creeps[name];
    }
  }

  makeCreep(CreepRole.HARVESTER, 1);
  makeCreep(CreepRole.UPGRADER, 3);
  makeCreep(CreepRole.BUILDER, 4);

  // Process current creeps
  for (let name in Game.creeps) {
    const creep = Game.creeps[name];
    if (creep.memory.role === CreepRole.HARVESTER) {
      runAsHarvester(creep);
    }
    if (creep.memory.role === CreepRole.UPGRADER) {
      runAsUpgrader(creep);
    }
    if (creep.memory.role === CreepRole.BUILDER) {
      runAsBuilder(creep);
    }
  }
});
